@rem = '--*-Perl-*--';
@rem = '
@echo off
rem -------------------------------------------------------------------
rem   MATLAB root directory
rem -------------------------------------------------------------------
rem   Change this if you move MATLAB without reinstalling.
set MATLAB=%MATLAB%

rem ------------- Do not modify anything below this line --------------
rem
rem ###################################################################
rem # Set up script location
rem ###################################################################
rem # Allow the full pathname (or current dir.) without an extension,
rem # allow the full pathname (or current dir.) with the .bat extension,
rem # use the installed location of mex.bat,
rem # or allow users to add mex.bat to the path and call mex.
set SCRIPTLOC=%0.bat
if not exist %SCRIPTLOC% set SCRIPTLOC=%0
if not exist %SCRIPTLOC% set SCRIPTLOC=%MATLAB%\bin\mex.bat
if exist %SCRIPTLOC%\* set SCRIPTLOC=%MATLAB%\bin\mex.bat
if not exist %SCRIPTLOC% set SCRIPTLOC=-S %0.bat

rem ###################################################################
rem # Set up perl location
rem ###################################################################
rem # Either use the installed location of perl,
rem # or hope it is on the DOS path.
set PERLLOC=%MATLAB%\bin\perl.exe
if not exist %PERLLOC% set PERLLOC=perl

rem ###################################################################
rem # Set up script arguments (to avoid %9 DOS batch file limit)
rem ###################################################################
rem # Put all arguments into environment variable ARGS.  This
rem # allows us to bypass the 9 parameter limit, but disallows
rem # -DVAR=VALUE and LINKER=mylinker constructions.  These cases
rem # will need to be handled in the mexopts file.
set MEXARGS= 
:getarg
set MEXARGS=%MEXARGS% %1
shift
if not "%1"=="" goto getarg

rem ###################################################################
rem # Call Perl with the script name as an argument
rem ###################################################################
set errlvl=
%PERLLOC% %SCRIPTLOC% %MEXARGS%
if errorlevel 1 set errlvl=1

rem ###################################################################
rem # Cleanup environment and exit
rem ###################################################################
set SCRIPTLOC=
set PERLLOC=
set MEXARGS=
goto endofperl
rem -------------------------------------------------------------------

@rem ';
#######################################################################
# Begin perl code
#######################################################################
#
# MEX builds MEX-files.
#
# $Revision: 1.11 $  $Date: 1996/11/07 15:22:13 $
#
#   Functional overview:
#   1. Define subroutines
#   2. Initialize variables
#   3. Parse input arguments & determine MEX-file name
#   4. Source options file
#   5. Determine final values
#   6. Compile source files
#   7. Perform prelink tasks
#   8. Link files
#
#######################################################################
# Define subroutines
#######################################################################

sub describe {
# Describe:  Issues messages.  This way lengthy messages do not clutter
#            up the main body of code.
    local($_) = $_[0];
 DESCRIPTION: {
     /^help$/ && print(<<"end_help") && last DESCRIPTION;
 MEX: Create a MEX-file from source code.

  MEX is a script which calls a compiler with the correct compiler and
  linker switches to create a MEX-file from source code.
  The script's execution is affected by both command-line arguments and
  a batch file, $OPTFILE_NAME, which contains all compiler-specific
  information necessary for the script's execution.  This batch file is
  searched for first in the current directory, and then the same directory
  as \"mex.bat\".  The -f switch allows it to be directly specified.

  Switches:
  -argcheck        Perform argument checking on MATLAB API fundtions
  -c               Compile only; do not link
  -D<name>         Define preprocessor symbol <name>
  -f <file>        Use <file> as the options file
  -g               Debug, i.e. use COMPDEBUGFLAGS and LINKDEBUGFLAGS
  -h[elp]          Help; print this message
  -I<pathname>     Include <pathname> in the compiler include path
  -output <name>   Name resulting MEX-file <name>.dll
  -O               Optimize, i.e. use COMPOPTIMFLAGS and LINKOPTIMFLAGS
  -U<name>         Undefine symbol <name>
  -V4              Compile MATLAB 4 compatible MEX-file
  -v               Verbose; print all compiler settings

end_help
     /^usage$/ && print(<<"end_usage") && last DESCRIPTION;
  Usage:  mex [ -argcheck -c -Dname -f file -g -h[elp] -Ipathname
                -output resultname -O -Uname -V4 -v ]
                sourcefile... [ objectfile ]...  [ library ]...

  Type 'mex -h' for more information, or consult the MATLAB API Guide manual.

end_usage
     /^general_info$/ && print(<<"end_general_info") && last DESCRIPTION;
 This is mex, (c) 1996 The MathWorks, Inc.

end_general_info
     /^wrong_options_file$/ && print(<<"end_wrong_options_file") && last DESCRIPTION;
    Warning: An invalid options file name was specified.

        Defaulting to $OPTFILE_NAME options file.
        Use mex -v to see what settings are used.

end_wrong_options_file
     /^no_options_file$/ && print(<<"end_no_options_file") && last DESCRIPTION;
    Sorry! No options file was found for mex.  The mex script must
           be able to source a options file to define compiler flags
           and other settings.  This options file is normally found in
           the same directory as this script:
           $script_directory\\$OPTFILE_NAME.
           If you have copied this script, please make sure to copy
           the options file as well.  Otherwise, please check to make
           sure that your installation  is complete and includes this
           file.

           Please contact:

               MathWorks Technical Support

           for further assistance.

end_no_options_file
     /^invalid_options_file$/ && print(<<"end_invalid_options_file") && last DESCRIPTION;
    Sorry! An options file for MEX.BAT was found and sourced, but the
           value for 'COMPILER' was not set.  This could mean that the
           value is not specified within the options file, or it could
           mean that there is a syntax error within the file.

           Please contact:

               MathWorks Technical Support

           for further assistance.

end_invalid_options_file
     /^final_options$/ && print(<<"end_final_options") && last DESCRIPTION;
$sourced_msg
----------------------------------------------------------------
->    Options file           = $OPTFILE_NAME
      MATLAB                 = $MATLAB
->    COMPILER               = $COMPILER
->    Compiler flags:
         COMPFLAGS           = $COMPFLAGS
         OPTIMFLAGS          = $OPTIMFLAGS
         DEBUGFLAGS          = $DEBUGFLAGS
         arguments           = $ARG_FLAGS
->    Pre-linking commands   = $PRELINK_CMDS
->    LINKER                 = $LINKER
->    Link directives:
         LINKFLAGS           = $LINKFLAGS
         Name directive      = $NAME_OUTPUT
         File link directive = $LINK_FILE
         Lib. link directive = $LINK_LIB
----------------------------------------------------------------

end_final_options
     /^file_not_found$/ && print(<<"end_file_not_found") && last DESCRIPTION;
  $cmd_name:  $filename not a normal file or does not exist.

end_file_not_found
     do {
         print "Internal error: Description for $_[0] not implemented\n";
         last DESCRIPTION;
     };
 }
}

sub find_options_file {
  if (-e ".\\$OPTFILE_NAME") {
    chop($source_dir = `cd`);
  } elsif (-e "$script_directory\\$OPTFILE_NAME") {
    $source_dir = "$script_directory";
  } else {
    &describe("no_options_file");
    &expire("Can not continue without options file.");
  }   
  $OPTFILE_NAME = "$source_dir\\$OPTFILE_NAME";
  $sourced_msg = "-> Default options filename found in $source_dir";
}

sub expire {
# Expire: Issue message and exit.  This is like "die" except that
#         it cleans up intermediate files before exiting.
#         &expire("normally") exits normally (doesn't die).

#   Clean up compiled files, unless we're only compiling
    foreach $filename (@FILES_TO_REMOVE) {
        unlink "$filename" if (-e "$filename");
    }
    die "\n  $cmd_name: $_[0]\n\n" unless ($_[0] eq "normally");
    exit(0);
}

#######################################################################
# Initialize variables
#######################################################################

$| = 1;                         # Force immediate output flushing

# Correct how the $cmd_name variable looks so that it is presentable
# to DOS users (i.e., trade / with \).
($cmd_name) = $0;
($cmd_name) =~ s/\//\\/g;
($cmd_name) =~ tr/a-z/A-Z/;

($script_directory) = ($cmd_name =~ /(.*)\\.*/);
$script_directory = '.' if (!($script_directory));

$OPTFILE_NAME = "mexopts.bat";
$sourced_msg = 'none';
$mex_include_dir = "extern\\include";

# Trap case where an invalid options file is used, by checking the
# value of the compiler
$COMPILER = "none";

# $$ should be the pid, but this is not defined for Windows perl
# We'll use a random integer instead.  This is only an issue
# if you build more than one mex file in the same directory at
# the same time, and this way there's a pretty low chance of
# failure.
srand;
$$ = int(rand(10000));
# Create a unique name for the created import library
$ENV{'LIB_NAME'} = "_lib$$";

#######################################################################
# Parse input arguments & determine MEX-file name from 1st source file
#######################################################################

for (;$_=shift(@ARGV);) {
    # Perl-style case construct
  ARGTYPE: {
      /^-argcheck$/ && do {
          $ARG_FLAGS = "$ARG_FLAGS -DARGCHECK";
          $argcheck = "yes";
          last ARGTYPE;
      };
      /^-c$/ && do {
          $compile_only = "yes";
          last ARGTYPE;
      };
      /^-[DIU]\S*$/ && do {
          $ARG_FLAGS = "$ARG_FLAGS $_";
          last ARGTYPE;
      };
      /^-f$/ && do {
          $filename = shift(@ARGV);
          if (!(-e $filename)) {
              &describe("file_not_found");
              &describe("wrong_options_file");
          } else {
              $OPTFILE_NAME =  "$filename";
              $sourced_msg = '-> Options file specified on command line';
          }
          last ARGTYPE;
      };
      /^-g$/ && do {
          $debug = "yes";
          last ARGTYPE;
      };
      /^[-\/](h(elp)?)|\?$/ && do {
          &describe("help");
          &expire("normally");
          last ARGTYPE;
      };
      /^-inline$/ && do {
          $inline = "yes";
          last ARGTYPE;
      };
      /^-output$/ && do {
          $ENV{'MEX_NAME'}=shift(@ARGV);
          last ARGTYPE;
      };
      /^-O$/ && do {
          $optimize = "yes";
          last ARGTYPE;
      };
      /^-v$/ && do {
          &describe("general_info");
          $verbose = "yes";
          last ARGTYPE;
      };
      /^-V4$/ && do {
          $ARG_FLAGS = "$ARG_FLAGS -DV4_COMPAT";
          last ARGTYPE;
      };
      /^\S*=\S*$/ && do {
          push(@ASSIGNMENTS, $_);
          last ARGTYPE;
      };
      /^-.*$/ && do {
          &describe("usage");
          &expire("Unrecognized switch: $_.");
          last ARGTYPE;
      };
      /^\S*$/ && do {
          if (!(-e "$_")) {
              &expire("'$_' not found.");
          }
          push(@FILES, $_);
          last ARGTYPE;
      };
  };
}

# get mex-file name and source file type from 1st file argument
($derived_name, $EXTENSION) = ($FILES[0] =~ /(\w*)\.(\w*)$/);
$ENV{'MEX_NAME'} = $derived_name if (!($ENV{'MEX_NAME'}));
$fortran = "yes" if ($EXTENSION eq 'f' || $EXTENSION eq 'F');

#######################################################################
# Source options file
#######################################################################

# If we don't already know what to use as the options file, search
# default locations.
if ($sourced_msg eq 'none') {
  &find_options_file;
}

# The DOS batch language is too limited for our purposes.
# Instead, we will parse the batch files on our own.
open (OPTIONSFILE, $OPTFILE_NAME) || expire("Can't open $OPTFILE_NAME");
while ($_ = <OPTIONSFILE>) {
    chop;
    next if (!(/^\s*set /));      # Ignore everything but set commands
    s/^\s*set //;                 # Remove "set " command itself
    s/\\$//g;                     # Remove trailing \'s
    s/\\/\\\\/g;                  # Escape all other \'s with another \
    s/%(\w+)%/'.\$ENV{'\1'}.'/g;  # Replace %VAR% with $ENV{'VAR'}
    s/%%/%/g;                     # Replace %%s with %s
    $perlvar = '$'.$_."';";
    $perlvar =~ s/=/='/;
    $dosvar = '$ENV{'."'".$_."';";
    $dosvar =~ s/=/'}='/;
    eval($perlvar);
    eval($dosvar);
}

&describe("final_options") if $verbose;

&describe("invalid_options_file") if ($COMPILER eq "none");

#######################################################################
# Determine final values
#######################################################################

# Decide how to optimize or debug
if (! $debug) {                                  # Normal case
    $FLAGS = "$OPTIMFLAGS";
    $LINKFLAGS = "$LINKFLAGS $LINKOPTIMFLAGS";
} elsif (! $optimize) {                          # Debug; don't optimize
    $FLAGS = "$DEBUGFLAGS";
    $LINKFLAGS = "$LINKFLAGS $LINKDEBUGFLAGS";
} else {                                         # Debug and optimize
    $FLAGS = "$DEBUGFLAGS $OPTIMFLAGS";
    $LINKFLAGS = "$LINKFLAGS $LINKOPTIMFLAGS $LINKDEBUGFLAGS";
}
# Include the simulink include directory if it exists
#
if (-e "$MATLAB\\simulink\\include") {
  $simulink_inc_dir = "-I$MATLAB\\simulink\\include";
} else {
  $simulink_inc_dir = "";
 }

$FLAGS = "-I$MATLAB\\$mex_include_dir $simulink_inc_dir $FLAGS" if (!($fortran));

# If there are no files, then exit.
if (@FILES == 0) {
    &describe("usage");
    &expire("No file names given.");
}

# If we are checking arguments, add $MATLAB/extern/src/mwdebug.c
# to source file list.
push(@FILES, "$MATLAB\\extern\\src\\mwdebug.c") if ($argcheck eq "yes");

#######################################################################
# Compile source files
#######################################################################

# Loop over @FILES to compile each file.  Keep files we actually
# compile separate from the ones we don't compile but need to link.
# This way, we can later clean up all .obj files we created.
for (;$_=shift(@FILES);) {
    ($FILENAME, $EXTENSION) = (/(\w+)\.(\w*)$/);
  FILETYPE: {
      /^\S*\.(c|f|cc|cxx|cpp|for)$/i && do {
          $ENV{'ARGS'} = "$ARG_FLAGS $COMPFLAGS $FLAGS $_";
          print "--> $COMPILER $ENV{'ARGS'}\n\n" if $verbose;
          $messages = `$COMPILER %ARGS%`;

          push(@FILES_TO_LINK,   "$LINK_FILE $FILENAME.obj");
          push(@FILES_TO_REMOVE, "$FILENAME.obj") unless $compile_only;

          # Check for error; $? doesn't work, so check for resulting file
          if (!(-e "$FILENAME.obj")) {
              print "$messages";
              &expire("Compile of '$_' failed.");
          }
          print $messages if $verbose;
          last FILETYPE;
      };
      /^\S*\.(lib|dll)$/i && do {
          push(@FILES_TO_LINK,"$LINK_LIB $_");
          last FILETYPE;
      };
      /^\S*$/ && do {
          push(@FILES_TO_LINK,"$LINK_FILE $_");
          last FILETYPE;
      };
  };
}

&expire("normally") if ($compile_only);

#######################################################################
# Perform prelink tasks
#######################################################################
#
# Note that error checking is not possible; we don't get a return
# status, and there's no way of knowing a priori what each task is
# supposed to do.
(@PRELINK_CMDS) = split(/;/,$PRELINK_CMDS);
while ($PRELINK_CMDS = shift(@PRELINK_CMDS)) {
    # Skip if $PRELINK_CMDS is only whitespace
    next if (!($PRELINK_CMDS =~ /\S/));

    print "\n--> $PRELINK_CMDS\n\n" if $verbose;
    $messages = `$PRELINK_CMDS`;
    print $messages if $verbose;
}

push(@FILES_TO_REMOVE, ("$ENV{'LIB_NAME'}1.lib", "$ENV{'LIB_NAME'}2.lib"));
push(@FILES_TO_REMOVE, ("$ENV{'LIB_NAME'}1.exp", "$ENV{'LIB_NAME'}2.exp"));

#######################################################################
# Link files
#######################################################################
# 
# NAME_OUTPUT always goes in the list, but it may be blank (in which
# case it's harmless to put it in).  Leaving the variable blank is
# equivalent to assuming that the output will be named after the
# first object file in the list.
$ENV{'ARGS'} = "$LINKFLAGS $NAME_OUTPUT @FILES_TO_LINK";

print "\n--> $LINKER $ENV{'ARGS'}\n\n" if $verbose;
$messages = `$LINKER %ARGS%`;

# Check for error; $? doesn't work, so check for resulting file
if (!(-e "$ENV{'MEX_NAME'}.dll")) {
  if (!(-e "$ENV{'MEX_NAME'}.exe")) {
    print "$messages";
    &expire("Link of '$ENV{'MEX_NAME'}.dll' failed.");
  }
}
print $messages if $verbose;

#######################################################################
# Clean up files and exit
#######################################################################

&expire("normally");

#######################################################################
# End perl code
#######################################################################
__END__
:endofperl
if X%errlvl% == X1 perl -e "die \"\n\";"
